#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h> 
#include "lcd.h"
#include "i2cmaster.h"
#include "uart.h"

// ak chces posielat data na uart tak odkomentuj:
#define UART

// konstanta na prepocet na uT
#define mag_const 0.0855

// struktura na ulozenie vektora mag. pola
typedef struct
{
	float x, y, z;
} vector;


int main(void){

	char text[32];
	float azimut,pitch,roll;    // azimut a naklony

	//pole pre surove data
	unsigned char ACC_Data[6],Mag_Data[6];

	vector acc, mag; //inicializacia vektorov

	// inicializacia I2C
	i2c_init();

	//inicializacia UART
	uart_init(9600);

	// inicializacia LCD displeja
	lcd_init(LCD_DISP_ON_CURSOR_BLINK);
	lcd_puts("hello");



	//nastav akcelerometer
	i2c_start(0x30); 
	i2c_write(0x20); // CTRL_REG1_A
	i2c_write(0x27); // normal power mode, 50 Hz update, vsetky osi zapnute
	i2c_stop();



	//nastav magnetometer
	i2c_start(0x3C); 
	i2c_write(0x02); // MR_REG_M
	i2c_write(0x00); // continuous conversion mode
	i2c_stop();

	i2c_start(0x3C); 
	i2c_write(0x01); // CRB_REG_M
	i2c_write(0x40); //SENSITIVITY 855 LSB na 1 GAUSS (100uT) 
	i2c_stop();


while(1)
{

	//citaj data s akcelerometra
	i2c_start(0x30); 
	i2c_write(0xa8); 

	i2c_start(0x31); // read acc
	ACC_Data[0] = i2c_readAck();
	ACC_Data[1] = i2c_readAck();
	ACC_Data[2] = i2c_readAck();
	ACC_Data[3] = i2c_readAck();
	ACC_Data[4] = i2c_readAck();
	ACC_Data[5] = i2c_readNak();
	i2c_stop();

	acc.x = (int) (ACC_Data[1] << 8) + ACC_Data[0];
	acc.y = (int) (ACC_Data[3] << 8) + ACC_Data[2];
	acc.z = (int) (ACC_Data[5] << 8) + ACC_Data[4];




	// citaj data s magnetometra
	i2c_start(0x3C); // write mag
	i2c_write(0x03); // OUTXH_M

	i2c_start(0x3D); // read mag
	Mag_Data[0] = i2c_readAck();
	Mag_Data[1] = i2c_readAck();
	Mag_Data[2] = i2c_readAck();
	Mag_Data[3] = i2c_readAck();
	Mag_Data[4] = i2c_readAck();
	Mag_Data[5] = i2c_readNak();
	i2c_stop();

	// raw data z magnetometra
	mag.x = (int) (Mag_Data[0] << 8) + Mag_Data[1];
	mag.z = (int) (Mag_Data[2] << 8) + Mag_Data[3];
	mag.y = (int) (Mag_Data[4] << 8) + Mag_Data[5];


	// kalibracia magnetometra
	vector cal;
	cal.x=mag.x+7;
	cal.y=mag.y+134;
	cal.z=mag.z;

	// vypocitaj pitch a roll
	roll   = atan2(  acc.y ,  sqrt(acc.x*acc.x+acc.z*acc.z));  
	pitch  = atan2(  -acc.x ,  sqrt(acc.y*acc.y+acc.z*acc.z));

		// Vypocitaj sin a cos pre dalsi vypocet
	float cosRoll = cos(roll);
	float sinRoll = sin(roll);  
	float cosPitch = cos(pitch);
	float sinPitch = sin(pitch);
  
	  // Kompenzacia naklonu magnetometra
	float Xh = cal.x * cosPitch + cal.z * sinPitch;
	float Yh = cal.x * sinRoll * sinPitch + cal.y * cosRoll - cal.z * sinRoll * cosPitch;

	// vypocet azimutu, prepocet radianov na stupne 
	azimut = 180 + (atan2(Yh, Xh) * 180/M_PI);

	lcd_clrscr();
	// vypis na displej azimut
	sprintf(text,"Azimut:%.2f",(double)azimut);
	lcd_gotoxy(0,0);
	lcd_puts(text);

/*
	//vypis kalibrovany vektor
	sprintf(text,"%d %d %d",(int)cal.x,(int)cal.y,(int)cal.z);
	lcd_gotoxy(0,1);
	lcd_puts(text);
*/


	//vypis naklon
	sprintf(text,"X:%.1f Y:%.1f",pitch*180/M_PI,roll*180/M_PI);
	lcd_gotoxy(0,1);
	lcd_puts(text);


#ifdef UART
	//posli cez UART do PC
	sprintf(text,"%d;%d;%d;%d;%d;%d\n",(int)mag.x,(int)mag.y,(int)mag.z,(int)acc.x,(int)acc.y,(int)acc.z);
	uart_puts(text);
#endif

	//refresh displeja
	_delay_ms(300);
}

}

