#include "uart.h"

/******************************************************
    inicializacia
    - nastavenie baudRate - ako vstupna premenna
    - zapne vysielac a prijimac
    - 8 datovych a 1 stop bity
******************************************************/

void uart_init(unsigned int baud)
{
    short int ubrr = F_CPU/16/baud-1;

	//nastavenie prenosovej rychlosti v UBRR
	UBRRH = (unsigned char) (ubrr>>8);
	UBRRL = (unsigned char) ubrr;

	//zapni prijmac a vysielac
	UCSRB = (1<<RXEN)|(1<<TXEN);

	//nastav format ramca: 8data, 1stop bity
	UCSRC = (1<<URSEL)|(3<<UCSZ0);
}

/******************************************************
    odosle bajt (znak)
******************************************************/

void uart_putc(unsigned char data)
{
	//cakaj na vyprazdnenie buffera
	while (!( UCSRA & (1<<UDRE)));

	//vloz data do buffer-a a odosli
	UDR = data;
}

/******************************************************
    odosle retazec bajtov (znakov)
******************************************************/

void uart_puts(char *s)
{
    while (*s)
      uart_putc(*s++);
}

/******************************************************
    prijem bajt-u (znak)
******************************************************/

unsigned char uart_getc(void)
{
	//cakaj na prijatie dat
	while (!(UCSRA & (1<<RXC)));

	//zober data z buffer-a
	return UDR;
}

/******************************************************
    vycistenie buffer-a
******************************************************/

void uart_flush(void)
{
	unsigned char dummy;

	while ( UCSRA & (1<<RXC) ) dummy = UDR;
}
